local json = require("json")
local utils = require("utils")
local const = require("const")
local Set = require("set")
local resty_string = require("resty.string")
local hmac = require("resty.hmac")
local resty_random = require("resty.random")
local pl_pretty = require("pl.pretty")

local request_method = ngx.var.request_method

utils.check_login()
_, login_token = utils.check_permission("GatewayManage")

local function get_pk_from_TSL(tsl)
    local args = json.decode(tsl) 
    if args then
        return args["profile"]["productKey"]
    else
        return nil
    end
end

local function get_product_name_from_TSL(tsl)
    local args = json.decode(tsl) 
    if args then
        return args["profile"]["productName"]
    else
        return nil
    end
end

local function set_product_name_to_TSL(tsl, product_name)
    local args = json.decode(tsl)
    args["profile"]["productName"] = product_name

    return json.encode(args)
end


if "POST" == request_method then

    ngx.req.read_body()
    local args_json = ngx.req.get_body_data()
    args = json.decode(args_json)
    if (nil == args) then
        utils.error_exit(400)
    end

    local request = args["request"]
    if (request == nil) then
        utils.error_exit(400, json.encode("request is invalid"))
    end

    local apiVer = request["apiVer"] 
    if (apiVer ~= "0.6") then
        utils.error_exit(400, json.encode("apiVer is invalid"))
    end
    
    local params = args["params"]
    if(nil == params) then
        utils.error_exit(400, json.encode("params should not be null"))
    end
    
    local standardTSL = params["standardTSL"]
    local extendTSL = params["extendTSL"]
    local productName = params["productName"]

    local productKey = get_pk_from_TSL(standardTSL)

    if (productKey == nil) then
        productKey = get_pk_from_TSL(extendTSL)
    end

    utils.check_pk(productKey)
 
    if productName == nil then
        productName = get_product_name_from_TSL(standardTSL)
    else
        set_product_name_to_TSL(standardTSL, productName)
    end

    local ret_code = false
    -- set standard tsl to config-manager
    if (standardTSL) then
        ret_code = utils.set_data(const.data_key.tsl.standard_tsl_prefix..productKey, json.encode(standardTSL))
    end 
    -- set extend tsl to config-manager
    if (extendTSL) then
        ret_code = utils.set_data(const.data_key.tsl.extend_tsl_prefix..productKey, json.encode(extendTSL))
    end
    
    if(ret_code == true) then
        utils.error_exit(200, json.encode("succeed to import TSL"))
    else
        utils.error_exit(500, json.encode("failed to import TSL"))
    end

end
utils.error_exit(400, json.encode("unsupported request method"))

------ EOF ------
