local json = require("json")
local utils = require("utils")
local utils_device = require("utils_device")
local const = require('const')
local request_method = ngx.var.request_method
local ldbus = require "ldbus"

utils.check_login()
utils.check_permission("DeviceManage")

local function generate_random_num()
    math.randomseed(tostring(os.time()):reverse():sub(1, 7))
    return math.random(1,100)
end

local function get_pk_dn_from_interface(interface, prefix)
    local s = utils.split_str(interface, "_")
    local pk = string.sub(s[1], string.len(prefix)+1, string.len(s[1]))
    local dn = s[2]

    return pk, dn
end

local function generate_subscribe_rule(eventInfo, dbus_wkn_name)
    local rule = {}
    local int count = 0
    local content = {}
    rule["version"] = "1.0"

    for _, event in pairs(eventInfo) do 
        local item = {}
        local source = {}
        local target = {}
        source["type"] = "device"
        source["data"] = "/"..event["productKey"].."/"..event["deviceName"]
        item["source"] = source

        target["type"] = dbus_wkn_name
        target["context"] = "{\"qos\":0}"
        item["target"] = target

        item["topicFilter"] = "all" 
        table.insert(content, item)
        count = count + 1
    end

    if count == 0 then
        count = count + 1
        local item = {}
        local source = {}
        local target = {}
        source["type"] = "device"
        source["data"] = "#"
        item["source"] = source

        target["type"] = dbus_wkn_name
        target["context"] = "{\"qos\":0}"
        item["target"] = target

        item["topicFilter"] = "all" 
        table.insert(content, item)
    end
    
    rule["content"] = content
    return count, rule
end

function call_message_route_method(conn, rule, method)
    local msg = assert ( ldbus.message.new_method_call ( const.subscribe.wkn,  const.subscribe.obj, const.subscribe.wkn , method) , "Message Null" )
    local iter = ldbus.message.iter.new ( )
    msg:iter_init_append ( iter )
    assert ( iter:append_basic ( json.encode(rule) ) , "Out of Memory" )
    local reply = assert ( conn:send_with_reply_and_block ( msg ) )
 
    return reply
end

function receive (eventInfo)
    local prefix = "iot.device.id"
    local postfix = "_"
    local cur_worker_pid = ngx.worker.pid()
    local wkn = "linkedge.webconsole.a"..cur_worker_pid..generate_random_num()
    
    utils.log_error ( "Listening for signals, pid: " ..cur_worker_pid.." -->:  "..wkn)

	local conn = assert ( ldbus.connection.open ( const.linkedge.bus_addr ) )

    if (ldbus.bus.register(conn, "register failed"))
    then
        utils.log_error("register succeed.")
    else
        utils.log_error("failed to register dbus")
    end
    assert ( ldbus.bus.request_name ( conn , wkn , { replace_existing = true } ) )
    local count, rule = generate_subscribe_rule(eventInfo, wkn)
    if count == 0 then
        conn:close()
        utils.error_exit(400, "event info is invalid")
    end

    call_message_route_method(conn, rule, const.subscribe.interface_sub)

    while conn:read_write ( 1*1000 ) do
        while true do
		local msg = conn:pop_message ( )
        if msg == nil then
            break
        end
        local res = '{"code": 204, "message": "heartbeats", "data": null}'
        if msg and msg:get_type ( ) == "signal" and msg:get_interface( ) ~= "org.freedesktop.DBus" then
			local iter = ldbus.message.iter.new ( )
			assert ( msg:iter_init ( iter ) , "Message has no parameters" )
			assert ( iter:get_arg_type ( ) == ldbus.types.string , "Argument is not a string" )
			local sigvalue = iter:get_basic ()
            local msg_type = msg:get_member()
            local pk, dn = get_pk_dn_from_interface(msg:get_interface(), prefix) 
            local ret_table = {}
            local ret_table_data = {}
            local ret_table_data_items = {}
            ret_table_data_items["productKey"] = pk
            ret_table_data_items["deviceName"] = dn
            ret_table_data_items["eventCode"] = ""
            ret_table_data_items["iotId"] = ""
            ret_table_data_items["batchId"] = ""
            ret_table_data_items["timestamp"] = ""
            ret_table_data_items["eventName"] = msg_type
            ret_table_data_items["eventType"] = msg_type 
            ret_table_data_items["eventBody"] = json.decode(sigvalue) 
            ret_table_data["timestamp"] = ""
            ret_table_data["items"] = ret_table_data_items
            ret_table["code"] = 200
            ret_table["message"] = "success"
            ret_table["data"] = ret_table_data
            
            res = json.encode(ret_table)
        end
        ok, err = ngx.say(res)
        ngx.flush(true)
        if ok
            then
                utils.log_error("send status: " ..ok)
            else
                utils.log_error("remote client offline now")
                call_message_route_method(conn, rule, const.subscribe.interface_unsub)
                conn:close()
                ngx.exit(ngx.HTTP_OK)
        end
        end
	end
    utils.log_error("clear dbus connection...")
end

if ("POST" == request_method) then
    local utils_header = require("utils_header")
    local params = utils_header.check_public_body_data()
    local eventInfo = params["eventInfo"]
    
    if eventInfo == nil or type(eventInfo) ~= "table" then
        utils.error_exit(400, "eventInfo should not be null")
    end

    receive (eventInfo)
end
