local json = require("json")
local utils = require("utils")
local utils_device = require("utils_device")

local request_method = ngx.var.request_method

utils.check_login()
utils.check_permission("DeviceManage")

function table_leng(t)
  local leng=0
  for k, v in pairs(t) do
    leng=leng+1
  end
  return leng;
end

if ("POST" == request_method) then
    local utils_header = require("utils_header")
    local params = utils_header.check_public_body_data()
    local driver_name = params["DriverName"]
    local pk = params["ProductKey"] 
    local tag = params["DeviceTags"]
    local cloud_state = params["CloudState"]
    local local_state = params["LocalState"]
    local page_size = tonumber(params["PageSize"])
    local cur_page = tonumber(params["CurrentPage"])
    local sort_method = params["SortMethod"]
    local dn =  params["DeviceName"]
    page_size = utils.check_page_size_allow_zero(page_size)
    if (pk and "" == string.gsub(pk, '%s', "")) then
        pk = nil -- treat space characters as empty
    end

    if (sort_method == nil) or ((sort_method ~= "TimeDesc") and (sort_method ~= "TimeAsc")) then
        sort_method = "TimeDesc"
    end

    local tag_list = {}
    if (tag and "string" == type(tag)) then
        -- convert to single tag as tag list
        table.insert(tag_list, tag)
    else
        tag_list = tag
    end

    utils.check_current_page(cur_page)

    local dev_param = {}
    if local_state == "Online" then
        dev_param["deviceState"] = "online" 
    elseif local_state == "Offline" then
        dev_param["deviceState"] = "offline"
    end

    if pk ~= nil and string.len(pk) > 0 then
        dev_param["productKey"] = pk
    end

    if driver_name ~= nil and string.len(driver_name) > 0 then
        dev_param["driverName"] = driver_name
    end

    if cloud_state == "Inactive" then 
        dev_param["activation"] = "notActivated" 
    elseif cloud_state == "ActivationFailed" then 
        dev_param["activation"] = "activationFailed"
    elseif cloud_state == "Online" then 
        dev_param["deviceCloudState"] = "online" 
        dev_param["activation"] = "activated"
    elseif cloud_state == "Offline" then 
        dev_param["deviceCloudState"] = "offline"
        dev_param["activation"] = "activated"
    end

    local cmd = nil
    if table_leng(dev_param) == 0 then
        cmd = ""
    else
        cmd = json.encode(dev_param)
    end
  
    if dn ~= nil and type(dn) == "string" and dn ~= "" and
        pk ~= nil and type(pk) == "string" then
        local dev_list = {}
        table.insert(dev_list, pk.."_"..dn)
        total, authed_list = utils_device.get_dev_list_by_page( dev_list, page_size, cur_page, utils_device.get_device_info_by_list, sort_method)
    else 
        total, authed_list = utils_device.get_authed_device_list_by_json(cmd, tag_list, page_size, cur_page, sort_method)
    end 
    res = {}
    res["TotalNum"] = total
    res["PageSize"] = page_size
    res["CurrentPage"] = cur_page
    res["List"] = authed_list
    res_raw = json.encode(res)
    utils.error_exit(200, res_raw)
end

utils.error_exit(400, "unsupported request method")
