local json = require("json")
local utils = require("utils")

utils.check_login()
utils.check_permission("DriverManage")

local request_method = ngx.var.request_method
if request_method ~= "POST" then
    return utils.error_exit(405, "unsupported request method: " .. request_method)
end

local utils_header = require("utils_header")
local params = utils_header.check_public_body_data()
local driver_id = params["DriverId"]
local service = params["Service"]
if type(driver_id) ~= "string" or #driver_id == 0 then
    return utils.error_exit(400, "DriverId is not string or is empty")
end
if type(service) ~= "table" then
    return utils.error_exit(400, "Service is not JSON object")
end

local service_name = service["Name"]
if type(service_name) ~= "string" or #service_name == 0 then
    return utils.error_exit(400, "Service.Name is not string or is empty")
end

local service_args = service["Args"]
local ret, err = utils.call_driver_service(driver_id, service_name, service_args)
if not ret then
    return utils.error_exit(500, err)
end

local res = {
    code = 200,
    message = "success",
    data = {}
}
res.data["Code"] = ret["code"]
res.data["Data"] = ret["data"]
ngx.say(json.encode(res))