local const = require("const")
local json = require("json")
local utils = require("utils3")

local shadow = {}

function shadow:new(o, product, device)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    self.product = product
    self.device = device
    return o
end

function shadow:get_properties(identifiers)
    local params = { params = identifiers }
    local id = self.product .. "_" .. self.device
    local well_known_name = const.cmp.wkn_prefix .. id
    local path = const.cmp.obj_prefix .. id
    local interface = const.cmp.inf_prefix .. id
    local ret, err, cause = utils.dbus_call_edge(
            well_known_name, path, interface,
            "callServices", "get", json.encode(params))
    if not ret then
        return false, err, cause
    end

    local ok, res = pcall(json.decode, ret[1])
    if not ok or type(res) ~= "table" then
        return false, const.MESSAGE_INVALIDREPLY, "not a JSON object"
    end
    if res["code"] ~= 0 then
        local msg = { code = res["code"], message = res["message"] }
        return false, const.SERVICE_THING_CALL, json.encode(msg)
    end
    return true, res["params"], nil
end

function shadow:set_properties(properties)
    local params = { params = properties }
    local id = self.product .. "_" .. self.device
    local well_known_name = const.cmp.wkn_prefix .. id
    local path = const.cmp.obj_prefix .. id
    local interface = const.cmp.inf_prefix .. id
    local ret, err, cause = utils.dbus_call_edge(
            well_known_name, path, interface,
            "callServices", "set", json.encode(params))
    if not ret then
        return false, err, cause
    end

    local ok, res = pcall(json.decode, ret[1])
    if not ok or type(res) ~= "table" then
        return nil, const.MESSAGE_INVALIDREPLY, "not a JSON object"
    end
    if res["code"] ~= 0 then
        local msg = { code = res["code"], message = res["message"] }
        return false, const.SERVICE_THING_CALL, json.encode(msg)
    end
    return true, res["params"], nil
end

function shadow:call_service(name, args)
    local params = { params = args }
    local id = self.product .. "_" .. self.device
    local well_known_name = const.cmp.wkn_prefix .. id
    local path = const.cmp.obj_prefix .. id
    local interface = const.cmp.inf_prefix .. id
    local ret, err, cause = utils.dbus_call_edge(
            well_known_name, path, interface,
            "callServices", name, json.encode(params))
    if not ret then
        return false, err, cause
    end

    local ok, res = pcall(json.decode, ret[1])
    if not ok or type(res) ~= "table" then
        return nil, const.MESSAGE_INVALIDREPLY, "not a JSON object"
    end
    if res["code"] ~= 0 then
        local msg = { code = res["code"], message = res["message"] }
        return false, const.SERVICE_THING_CALL, json.encode(msg)
    end
    local res_params = res["params"]
    if res_params then
        if res_params["code"] ~= 0 then
            return false, const.MESSAGE_FAILEDREPLY, res_params["message"]
        end
        return true, res_params["data"], nil
    end
    return true, nil, nil
end

return shadow
