#!/usr/bin/env python3
# -*- coding: utf-8 -*-


# ====#====#====#====
# Author: jys
# File: store
# Time: 2019/10/8
# ====#====#====#====

import logging
import time
from httpClient import HttpClient

_logger = logging.getLogger(__name__)

class Tsdb_Store(object):
	def __init__(self, host='localhost', port=8242):
		
		hostUrl = host + ':' + str(port)
		
		self.client = HttpClient(hostUrl)
	
	def _formatProperty(self, propertyInfo, propertyTags = None):
		dataDict = {}
		singleFlag = True
		
		dataDict["metric"] = propertyInfo["propertyName"]
		dataDict["tags"] = {
			"_productKey": propertyInfo["pk"],
			"_deviceName": propertyInfo["dn"],
			"_dataType": "property"
		}
		
		if(None !=propertyTags):
			dataDict["tags"].update(propertyTags)
		
		dataDict["timestamp"] = propertyInfo["propertyData"]["time"]
		if (isinstance(propertyInfo["propertyData"]["value"], dict)):
			dataDict["fields"] = propertyInfo["propertyData"]["value"]
			singleFlag = False
		elif (isinstance(propertyInfo["propertyData"]["value"], list)):
			_logger.warning("influxdb not support list type")
			singleFlag = False
		else:
			dataDict["value"] = propertyInfo["propertyData"]["value"]
		
		return singleFlag, dataDict
	
	def _formatEvent(self, eventInfo, eventTages = None):
		dataDict = {}
		
		dataDict["metric"] = eventInfo["eventName"]
		dataDict["tags"] = {
			"_productKey": eventInfo["pk"],
			"_deviceName": eventInfo["dn"],
			"_dataType": "event",
		}
		
		if (None != eventTages):
			dataDict["tags"].update(eventTages)
		
		dataDict["timestamp"] = eventInfo["eventData"]["time"]
		dataDict["fields"] = eventInfo["eventData"]["value"]
		
		return dataDict
	
	def storeProperty(self, propertyList):
		if (len(propertyList) == 0):
			return
		
		dataDict_single = []
		dataDict_mul = []
		
		try:
			for item in propertyList:
				flag, dataDict = self._formatProperty(item)
				if(flag):
					dataDict_single.append(dataDict)
				else:
					dataDict_mul.append(dataDict)

			_logger.info("store property cnt: %s", len(propertyList))
			
			if(len(dataDict_single)):
				self.client.httpPostJson('/api/put', dataDict_single)
			
			if(len(dataDict_mul)):
				self.client.httpPostJson('/api/mput', dataDict_mul)
		except:
			_logger.exception("Err")
	
	def storeEvent(self, eventList):
		if (len(eventList) == 0):
			return
		
		try:
			dataDict = [self._formatEvent(item) for item in eventList]
			_logger.info("store event cnt data: %s", len(eventList))
			self.client.httpPostJson('/api/mput', dataDict)
		
		except:
			_logger.exception("Err")
